using UnityEngine;
using System.Collections;
using System;
using UnityEngine.Events;

namespace Mod
{
	public class Mod
	{
		public static void Main()
		{
			CategoryBuilder.Create("BBQ's Research Facility", "Prepare for unforeseen consequences.", ModAPI.LoadSprite("category.png"));
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"), //item to derive from
					NameOverride = "[ Half-Life / Protagonist ] Gordon Freeman(HEV Suit)", //new item name with a suffix to assure it is globally unique
					DescriptionOverride = "...", //new item description
					CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"), //new item category
					ThumbnailOverride = ModAPI.LoadSprite("Thumb1.png"), //new item thumbnail (relative path)
					AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
					{   		
						var person = Instance.GetComponent<PersonBehaviour>();
						
						var prop = UnityEngine.Object.Instantiate<PhysicalProperties>(person.Limbs[0].PhysicalBehaviour.Properties);
						person.SetBodyTextures(ModAPI.LoadTexture("GordonFreemanHEV.png"), ModAPI.LoadTexture("GordonFreemanHEV.png"), null, 2);
						
						person.Limbs[0].SkinMaterialHandler.renderer.sprite = ModAPI.LoadSprite("Gordonhead.png", 2);
						person.Limbs[0].SkinMaterialHandler.renderer.material.SetTexture("_FleshTex", ModAPI.LoadTexture("FleshHead.png"));
						person.Limbs[0].SkinMaterialHandler.renderer.material.SetTexture("_BoneTex", ModAPI.LoadTexture("BoneHead.png"));
						
						person.BoneBreakClips = new AudioClip[]
						{
							ModAPI.LoadSound("HEV.wav"),
						};
						
						
						
						foreach (var body in person.Limbs)
						{
							body.PhysicalBehaviour.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
						}
									
							
						//load textures for each layer (see Human textures folder in this repository)
						var skin = ModAPI.LoadTexture("GordonFreemanHEV.png");
						var flesh = ModAPI.LoadTexture("2.png");
						var bone = ModAPI.LoadTexture("3.png");
			
						//get person
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 2);
					}
				}
			);	
			ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "[ Half-Life / Weapon ] Crowbar", //new item name with a suffix to assure it is globally unique
                    DescriptionOverride = "Better than Lightsaber", //new item description
                    CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumb2.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Crowbar.png", 1.75f);
                        Instance.FixColliders();
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
			);
			ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Physics Gun"),
                    NameOverride = "[ Half-Life 2 / Device ] Black Mesa Zero Point Energy Field Manipulator",
                    DescriptionOverride = "The Zero Point Energy Field Manipulator is not a toy, Alyx!",
                    CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumb3.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("GravityGun.png", 2f);
					  //Instance.FixColliders();
						Instance.GetComponent<PhysicalBehaviour>().ReflectsLasers = false;
					
						AudioSource AS = Instance.AddComponent<AudioSource>();
						AS.volume = 5000;
						AS.minDistance = 1;
						AS.maxDistance = 3;
						AS.loop = false;

						int index = 0;
						AudioClip[] AudioData = new AudioClip[]
						{
							ModAPI.LoadSound("PhysicsGun.wav")

						};
						
						Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
							AS.clip = AudioData[index];
							AS.Play();
							
	
							index++;
							if (index == AudioData.Length) index = 0;	
						};
						
						
						var childObject = new GameObject("eye");
						childObject.transform.SetParent(Instance.transform);
						childObject.transform.localPosition = new Vector3(0f, 0f);
						childObject.transform.localScale = new Vector3(1f, 1f);
						var childSprite = childObject.AddComponent<SpriteRenderer>();
						childSprite.sprite = ModAPI.LoadSprite("GravityGunLight.png" ,2f);
						childSprite.sharedMaterial = ModAPI.FindMaterial("VeryBright");
						childObject.GetComponent<SpriteRenderer>().sortingLayerName = "Bubbles";

						childSprite.color = new Color(1f, 1f, 1f, 0.025f);
						
					}
				}
			);
			ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Pulse Drum"),
                    NameOverride = "[ Half-Life 2: Episode 1 / Device ] Black Mesa Dark Energy Field Manipulator",
                    DescriptionOverride = "Dark Energy can make Gravity Gun overcharge.",
                    CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumb9.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("PhysicsGun.png", 2f);
					    Instance.FixColliders();
						Instance.GetComponent<PhysicalBehaviour>().ReflectsLasers = false;
					
						AudioSource AS = Instance.AddComponent<AudioSource>();
						AS.volume = 5000;
						AS.minDistance = 1;
						AS.maxDistance = 3;
						AS.loop = false;

						int index = 0;
						AudioClip[] AudioData = new AudioClip[]
						{
							ModAPI.LoadSound("GarryGun.wav")

						};
						
						Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
							AS.clip = AudioData[index];
							AS.Play();
							
	
							index++;
							if (index == AudioData.Length) index = 0;	
						};
						
						
						var childObject = new GameObject("eye");
						childObject.transform.SetParent(Instance.transform);
						childObject.transform.localPosition = new Vector3(0f, 0f);
						childObject.transform.localScale = new Vector3(1f, 1f);
						var childSprite = childObject.AddComponent<SpriteRenderer>();
						childSprite.sprite = ModAPI.LoadSprite("PhysicsGunLight.png" ,2f);
						childSprite.sharedMaterial = ModAPI.FindMaterial("VeryBright");
						childObject.GetComponent<SpriteRenderer>().sortingLayerName = "Bubbles";

						childSprite.color = new Color(1f, 1f, 1f, 0.025f);
						
					}
				}
			);
			ModAPI.Register(
			    new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Pistol"),
					NameOverride = "[ Half-Life 2 / Weapon ] USP Match",
					DescriptionOverride = "Combine's standard sidearm.",
					CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumb4.png"),
					AfterSpawn = (Instance) =>
					{
						ModAPI.KeepExtraObjects();
						
						var Slide = Instance.transform.Find("Slide");
                        Slide.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("PistolSlide.png", 4f);	
					    Slide.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
						Slide.GetComponent<SpriteRenderer>().sortingOrder = -1;
						
						
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Pistol.png", 4f);
						Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
						Instance.GetComponent<SpriteRenderer>().sortingOrder = -2;
						
						var DotLight = new GameObject("eye");
						DotLight.transform.SetParent(Slide.transform);
						DotLight.transform.localPosition = new Vector3(0f, 0f);
						DotLight.transform.localScale = new Vector3(1f, 1f);
						var DotLightSprite = DotLight.AddComponent<SpriteRenderer>();
						DotLightSprite.sprite = ModAPI.LoadSprite("Light.png" ,4f);
						DotLightSprite.sharedMaterial = ModAPI.FindMaterial("VeryBright");
						DotLight.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

						DotLightSprite.color = new Color(0f, 1f, 0f, 0.25f);
						
						var Hammer = new GameObject("eye");
						Hammer.transform.SetParent(Instance.transform);
						Hammer.transform.localPosition = new Vector3(0f, 0f);
						Hammer.transform.localScale = new Vector3(1f, 1f);
						var HammerSprite = Hammer.AddComponent<SpriteRenderer>();
						HammerSprite.sprite = ModAPI.LoadSprite("Hammer.png" ,4f);
						HammerSprite.sharedMaterial = ModAPI.FindMaterial("Sprites-Default");
						Hammer.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";

						HammerSprite.color = new Color(1f, 1f, 1f, 1f);
					
						
						var firearm = Instance.GetComponent<FirearmBehaviour>();
						
						Cartridge customCartridge = ModAPI.FindCartridge("50 AE");																		
						firearm.casingPosition = new Vector3(0f,0.1f);						
						foreach (var c in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
						}
                        Instance.AddComponent<BoxCollider2D>();
                        Instance.GetComponent<PhysicalBehaviour>().colliders = Instance.GetComponentsInChildren<Collider2D>();
						
						customCartridge.Recoil *= 5f; //change the recoil
						customCartridge.ImpactForce *= 0.7f; //change how much the bullet pushes the target

					    Instance.FixColliders();
						firearm.ShotSounds = new AudioClip[]
                        {
                            ModAPI.LoadSound("RiseAndShine.wav"),

                        };
			        }
                }
            );
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"), //item to derive from
					NameOverride = "[ Half-Life / Protagonist ] Gordon Freeman(Lab Coat)", //new item name with a suffix to assure it is globally unique
					DescriptionOverride = "...", //new item description
					CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"), //new item category
					ThumbnailOverride = ModAPI.LoadSprite("Thumb5.png"), //new item thumbnail (relative path)
					AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
					{   		
						var person = Instance.GetComponent<PersonBehaviour>();
						
						var prop = UnityEngine.Object.Instantiate<PhysicalProperties>(person.Limbs[0].PhysicalBehaviour.Properties);
						person.SetBodyTextures(ModAPI.LoadTexture("GordonFreemanHL1.png"), ModAPI.LoadTexture("GordonFreemanHL1.png"), null, 2);
						
						person.Limbs[0].SkinMaterialHandler.renderer.sprite = ModAPI.LoadSprite("Gordonhead.png", 2);
						person.Limbs[0].SkinMaterialHandler.renderer.material.SetTexture("_FleshTex", ModAPI.LoadTexture("FleshHead.png"));
						person.Limbs[0].SkinMaterialHandler.renderer.material.SetTexture("_BoneTex", ModAPI.LoadTexture("BoneHead.png"));
						
						
						
						
						
						foreach (var body in person.Limbs)
						{
							body.PhysicalBehaviour.Properties = ModAPI.FindPhysicalProperties("Human");
						}
									
							
						//load textures for each layer (see Human textures folder in this repository)
						var skin = ModAPI.LoadTexture("GordonFreemanHL1.png");
						var flesh = ModAPI.LoadTexture("2.png");
						var bone = ModAPI.LoadTexture("3.png");
			
						//get person
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 2);
					}
				}
			);	
			ModAPI.Register(
			    new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Submachine Gun"),
					NameOverride = "[ Half-Life 2 / Weapon ] MP7",
					DescriptionOverride = "Combine's standard weapon.",
					CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumb6.png"),
					AfterSpawn = (Instance) =>
					{
						ModAPI.KeepExtraObjects();
						
						var Slide = Instance.transform.Find("Slide");
                        Slide.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("MP76.png", 4f);	
					    Slide.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
						Slide.GetComponent<SpriteRenderer>().sortingOrder = -1;
						
						
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("MP7.png", 4f);
						Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
						Instance.GetComponent<SpriteRenderer>().sortingOrder = -2;
						
						
					
						
						var firearm = Instance.GetComponent<FirearmBehaviour>();
						
						Cartridge customCartridge = ModAPI.FindCartridge("50 BMG");																		
						firearm.casingPosition = new Vector3(0f,0.1f);						
						foreach (var c in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
						}
                        Instance.AddComponent<BoxCollider2D>();
                        Instance.GetComponent<PhysicalBehaviour>().colliders = Instance.GetComponentsInChildren<Collider2D>();
						
						customCartridge.Recoil *= 10f; //change the recoil
						customCartridge.ImpactForce *= 0.7f; //change how much the bullet pushes the target

					    Instance.FixColliders();
						firearm.ShotSounds = new AudioClip[]
                        {
                            ModAPI.LoadSound("Lambda.wav"),

                        };
			        }
                }
            );
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"), //item to derive from
					NameOverride = "[ Half-Life 2 / Antagonist ] Combine Elite", //new item name with a suffix to assure it is globally unique
					DescriptionOverride = "High-ranked specialized Combine Soldier", //new item description
					CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"), //new item category
					ThumbnailOverride = ModAPI.LoadSprite("Thumb7.png"), //new item thumbnail (relative path)
					AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
					{   		
						var person = Instance.GetComponent<PersonBehaviour>();
						var head = Instance.transform.Find("Head");
						
						GameObject Eyebigtime = new GameObject();
            			Eyebigtime.transform.parent = head;
            			Eyebigtime.transform.localPosition = new Vector3(0f, 0f);
            			Eyebigtime.transform.localScale = new Vector3(0.5f, 0.5f);

  						var eyebigtimeSprite = Eyebigtime.AddComponent<SpriteRenderer>();

  						eyebigtimeSprite.sprite = ModAPI.LoadSprite("CombineEliteEyelight.png");
						eyebigtimeSprite.material = ModAPI.FindMaterial("VeryBright");
            			eyebigtimeSprite.color = new Color(1f, 0f, 0f, 0.05f);
   						eyebigtimeSprite.sortingLayerName = "Foreground";
						
						foreach (var body in person.Limbs)
						{
							body.PhysicalBehaviour.Properties = ModAPI.FindPhysicalProperties("Flammable Metal");
						}
									
							
						//load textures for each layer (see Human textures folder in this repository)
						var skin = ModAPI.LoadTexture("CombineElite.png");
						var flesh = ModAPI.LoadTexture("2.png");
						var bone = ModAPI.LoadTexture("3.png");
			
						//get person
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 2);
					}
				}
			);	
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"), //item to derive from
					NameOverride = "[ Half-Life / ? ] G-MAN", //new item name with a suffix to assure it is globally unique
					DescriptionOverride = "	Rise and shine, Mr. Freeman. Rise and shine.", //new item description
					CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"), //new item category
					ThumbnailOverride = ModAPI.LoadSprite("Thumb8.png"), //new item thumbnail (relative path)
					AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
					{   		
						var person = Instance.GetComponent<PersonBehaviour>();
						var head = Instance.transform.Find("Head");
						
						GameObject Eyebigtime = new GameObject();
            			Eyebigtime.transform.parent = head;
            			Eyebigtime.transform.localPosition = new Vector3(0f, 0f);
            			Eyebigtime.transform.localScale = new Vector3(0.5f, 0.5f);

  						var eyebigtimeSprite = Eyebigtime.AddComponent<SpriteRenderer>();

  						eyebigtimeSprite.sprite = ModAPI.LoadSprite("GmanEye.png");
						eyebigtimeSprite.material = ModAPI.FindMaterial("VeryBright");
            			eyebigtimeSprite.color = new Color(1f, 1f, 1f, 0.025f);
   						eyebigtimeSprite.sortingLayerName = "Foreground";
						
						var prop = UnityEngine.Object.Instantiate<PhysicalProperties>(person.Limbs[0].PhysicalBehaviour.Properties);
						person.SetBodyTextures(ModAPI.LoadTexture("Gman.png"), ModAPI.LoadTexture("Gman.png"), null, 2);
						
						person.Limbs[0].SkinMaterialHandler.renderer.sprite = ModAPI.LoadSprite("GmanHead.png", 2);
						person.Limbs[0].SkinMaterialHandler.renderer.material.SetTexture("_FleshTex", ModAPI.LoadTexture("GmanHead.png"));
						person.Limbs[0].SkinMaterialHandler.renderer.material.SetTexture("_BoneTex", ModAPI.LoadTexture("GmanHead.png"));
						
					
						foreach (var body in person.Limbs)
						{
							body.PhysicalBehaviour.Properties = ModAPI.FindPhysicalProperties("Incredible");
						}
									
							
						//load textures for each layer (see Human textures folder in this repository)
						var skin = ModAPI.LoadTexture("Gman.png");
						var flesh = ModAPI.LoadTexture("Gman.png");
						var bone = ModAPI.LoadTexture("Gman.png");
			
						//get person
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 2);
					}
				}
			);	
			var discord = ModAPI.FindSpawnable("Discord");
            if (discord == null)
            {
                ModAPI.Register(
                    new Modification()
                    {
                        OriginalItem = ModAPI.FindSpawnable("Rod"),
                        NameOverride = "!!Discord!!",
                        DescriptionOverride = "BarBeQ's Official Discord",
                        CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"), //new item category
                        ThumbnailOverride = ModAPI.LoadSprite("discord.png"),
                        AfterSpawn = (Instance) =>
                        {
                            Utils.OpenURL("https://discord.gg/FnsvEVzKCs");
                            GameObject.Destroy(Instance);
                        }
                    }
                );
            }
			ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Shotgun"),
                    NameOverride = "[ Half-Life 2 ] SPAS-12",
                    DescriptionOverride = "",
                    CategoryOverride = ModAPI.FindCategory("BBQ's Research Facility"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumb10.png"),
                    AfterSpawn = (Instance) =>
                    {
                        ModAPI.KeepExtraObjects();

                        var SaxtonHaleSlide = Instance.transform.Find("Slide");
                        SaxtonHaleSlide.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("ShotgunSlide.png", 4f);
						SaxtonHaleSlide.GetComponent<SpriteRenderer>().color = Color.white;
						SaxtonHaleSlide.localScale = new Vector3(1f, 1f);
						SaxtonHaleSlide.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
						SaxtonHaleSlide.GetComponent<SpriteRenderer>().sortingOrder = -1;
						
                       
                       
                        var SaxtonHaleCover = Instance.transform.Find("Cover");
                        SaxtonHaleCover.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Cover.png", 4f);
						SaxtonHaleCover.GetComponent<SpriteRenderer>().color = Color.white;
						SaxtonHaleCover.localScale = new Vector3(1f, 1f);
						
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Shotgun.png", 4f);  
						Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
						Instance.GetComponent<SpriteRenderer>().sortingOrder = -2;

                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        firearm.BulletsPerShot = 7;
                          
                        firearm.InitialInaccuracy = 0.125f;
                        firearm.ShotSounds = new AudioClip[]
                        {
                              ModAPI.LoadSound("Shotgun.wav"),
                        };

                        Cartridge customCartridge = ModAPI.FindCartridge("50 BMG");
                        customCartridge.name = "SPAS-12";
                        customCartridge.Damage = 25f;
                        customCartridge.Recoil = 0.25f;
                        customCartridge.ImpactForce = 0.025f;
                        firearm.Cartridge = customCartridge;
					}
                }
            );
		}
	}
}